""" Access control testing
"""

from django.test import override_settings
from unittest.mock import patch
from tests.components.template_version_manager.fixtures.fixtures import (
    TemplateVersionManagerAccessControlFixtures,
)
from core_main_app.components.version_manager import (
    access_control as access_control_api,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request

fixture_template_vm = TemplateVersionManagerAccessControlFixtures()


class TestVersionManagerDisable(IntegrationBaseTestCase):
    """TestVersionManagerDisable"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_disable_user_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous disable user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.user1_tvm, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_disable_user_version_manager_with_access_right_raises_access_control_error(
        self,
    ):
        """test anonymous disable user version manager with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.user1_tvm, request=mock_request
            )

    def test_anonymous_disable_global_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous disable global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.global_tvm, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_disable_global_version_manager_with_access_right_raises_access_control_error(
        self,
    ):
        """test anonymous disable global version manager with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.global_tvm, request=mock_request
            )

    def test_user_can_disable_user_version_manager(self):
        """test user can disable user version manager

        Returns:

        """
        self.fixture.user1_tvm.is_disabled = False
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.disable(
            self.fixture.user1_tvm, request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm.is_disabled)

    def test_user_disable_global_version_manager_raises_access_control_error(
        self,
    ):
        """test user disable global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.global_tvm, request=mock_request
            )

    def test_user_disable_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test user disable other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.user2_tvm, request=mock_request
            )

    def test_staff_can_disable_user_version_manager(self):
        """test staff can disable user version manager

        Returns:

        """
        self.fixture.user1_tvm.is_disabled = False
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.disable(
            self.fixture.user1_tvm, request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm.is_disabled)

    def test_staff_can_disable_global_version_manager(self):
        """test staff can disable global version manager

        Returns:

        """
        self.fixture.global_tvm.is_disabled = False
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.disable(
            self.fixture.global_tvm, request=mock_request
        )
        self.assertTrue(self.fixture.global_tvm.is_disabled)

    def test_staff_disable_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test staff disable other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(
                self.fixture.user2_tvm, request=mock_request
            )

    def test_superuser_can_disable_user_version_manager(self):
        """test superuser can disable user version manager

        Returns:

        """
        self.fixture.user1_tvm.is_disabled = False
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable(
            self.fixture.user1_tvm, request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm.is_disabled)

    def test_superuser_can_disable_global_version_manager(self):
        """test superuser can disable global version manager

        Returns:

        """
        self.fixture.global_tvm.is_disabled = False
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable(
            self.fixture.global_tvm, request=mock_request
        )
        self.assertTrue(self.fixture.global_tvm.is_disabled)

    def test_superuser_can_disable_other_user_version_manager(self):
        """test superuser can disable other user version manager

        Returns:

        """
        self.fixture.user2_tvm.is_disabled = False
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable(
            self.fixture.user2_tvm, request=mock_request
        )
        self.assertTrue(self.fixture.user2_tvm.is_disabled)


class TestVersionManagerRestore(IntegrationBaseTestCase):
    """TestVersionManagerRestore"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_restore_user_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous restore user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.user1_tvm, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_restore_user_version_manager_with_access_right_raises_access_control_error(
        self,
    ):
        """test anonymous restore user version manager with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.user1_tvm, request=mock_request
            )

    def test_anonymous_restore_global_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous restore global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.global_tvm, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_restore_global_version_manager_with_access_right_raises_access_control_error(
        self,
    ):
        """test anonymous restore global version manager with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.global_tvm, request=mock_request
            )

    def test_user_can_restore_user_version_manager(self):
        """test user can restore user version manager

        Returns:

        """
        self.fixture.user1_tvm.is_disabled = True
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.restore(
            self.fixture.user1_tvm, request=mock_request
        )
        self.assertFalse(self.fixture.user1_tvm.is_disabled)

    def test_user_restore_global_version_manager_raises_access_control_error(
        self,
    ):
        """test user restore global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.global_tvm, request=mock_request
            )

    def test_user_restore_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test user restore other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.user2_tvm, request=mock_request
            )

    def test_staff_can_restore_user_version_manager(self):
        """test staff can restore user version manager

        Returns:

        """
        self.fixture.user1_tvm.is_disabled = True
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.restore(
            self.fixture.user1_tvm, request=mock_request
        )
        self.assertFalse(self.fixture.user1_tvm.is_disabled)

    def test_staff_can_restore_global_version_manager(self):
        """test staff can restore global version manager

        Returns:

        """
        self.fixture.global_tvm.is_disabled = True
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.restore(
            self.fixture.global_tvm, request=mock_request
        )
        self.assertFalse(self.fixture.global_tvm.is_disabled)

    def test_staff_restore_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test staff restore other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(
                self.fixture.user2_tvm, request=mock_request
            )

    def test_superuser_can_restore_user_version_manager(self):
        """test superuser can restore user version manager

        Returns:

        """
        self.fixture.user1_tvm.is_disabled = True
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore(
            self.fixture.user1_tvm, request=mock_request
        )
        self.assertFalse(self.fixture.user1_tvm.is_disabled)

    def test_superuser_can_restore_global_version_manager(self):
        """test superuser can restore global version manager

        Returns:

        """
        self.fixture.global_tvm.is_disabled = True
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore(
            self.fixture.global_tvm, request=mock_request
        )
        self.assertFalse(self.fixture.global_tvm.is_disabled)

    def test_superuser_can_restore_other_user_version_manager(self):
        """test superuser can restore other user version manager

        Returns:

        """
        self.fixture.user2_tvm.is_disabled = True
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore(
            self.fixture.user2_tvm, request=mock_request
        )
        self.assertFalse(self.fixture.user2_tvm.is_disabled)


class TestVersionManagerRestoreVersion(IntegrationBaseTestCase):
    """TestVersionManagerRestoreVersion"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_restore_version_user_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous restore version user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user1_template, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_restore_version_user_version_manager_with_access_right_raises_acl_error(
        self,
    ):
        """test anonymous restore version user version manager with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user1_template, request=mock_request
            )

    def test_anonymous_restore_version_global_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous restore version global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.global_template, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_restore_version_global_version_manager_with_access_right_raises_acl_error(
        self,
    ):
        """test anonymous restore version global version manager with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_can_restore_version_user_version_manager(self):
        """test user can restore version user version manager

        Returns:

        """
        self.fixture.user1_template.is_disabled = True
        self.fixture.user1_template.save_template()
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.restore_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id)
            not in self.fixture.user1_template.version_manager.disabled_versions
        )

    def test_user_restore_version_global_version_manager_raises_access_control_error(
        self,
    ):
        """test user restore version global version manager raises access control error

        Returns:

        """
        self.fixture.global_template.is_disabled = True
        self.fixture.global_template.save_template()
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_restore_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test user restore version other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_staff_can_restore_version_user_version_manager(self):
        """test staff can restore version user version manager

        Returns:

        """
        self.fixture.user1_template.is_disabled = True
        self.fixture.user1_template.save_template()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.restore_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id)
            not in self.fixture.user1_template.version_manager.disabled_versions
        )

    def test_staff_can_restore_version_global_version_manager(self):
        """test staff can restore version global version manager

        Returns:

        """
        self.fixture.global_template.is_disabled = True
        self.fixture.global_template.save_template()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.restore_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id)
            not in self.fixture.global_template.version_manager.disabled_versions
        )

    def test_staff_restore_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test staff restore version other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_superuser_can_restore_version_user_version_manager(self):
        """test superuser can restore version user version manager

        Returns:

        """
        self.fixture.user1_template.is_disabled = True
        self.fixture.user1_template.save_template()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id)
            not in self.fixture.user1_template.version_manager.disabled_versions
        )

    def test_superuser_can_restore_version_global_version_manager(self):
        """test superuser can restore version global version manager

        Returns:

        """
        self.fixture.global_template.is_disabled = True
        self.fixture.global_template.save_template()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id)
            not in self.fixture.global_template.version_manager.disabled_versions
        )

    def test_superuser_can_restore_version_other_user_version_manager(self):
        """test superuser can restore version other user version manager

        Returns:

        """
        self.fixture.user2_template.is_disabled = True
        self.fixture.user2_template.save_template()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore_version(
            self.fixture.user2_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user2_template.id)
            not in self.fixture.user2_template.version_manager.disabled_versions
        )


class TestVersionManagerDisableVersion(IntegrationBaseTestCase):
    """TestVersionManagerDisableVersion"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_disable_version_user_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous disable version user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user1_template, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_disable_version_user_version_manager_with_access_right_raises_acl_error(
        self,
    ):
        """test anonymous disable version user version manager with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user1_template, request=mock_request
            )

    def test_anonymous_disable_version_global_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous disable version global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.global_template, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_disable_version_global_version_manager_with_access_right_raises_acl_error(
        self,
    ):
        """test anonymous disable version global version manager with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_can_disable_version_user_version_manager(self):
        """test user can disable version user version manager

        Returns:

        """
        self.fixture.user1_template.is_current = False
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.disable_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id)
            in self.fixture.user1_template.version_manager.disabled_versions
        )

    def test_user_disable_version_global_version_manager_raises_access_control_error(
        self,
    ):
        """test user disable version global version manager raises access control error

        Returns:

        """
        self.fixture.global_template.is_current = False
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_disable_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test user disable version other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_staff_can_disable_version_user_version_manager(self):
        """test staff can disable version user version manager

        Returns:

        """
        self.fixture.user1_template.is_current = False
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.disable_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id)
            in self.fixture.user1_template.version_manager.disabled_versions
        )

    def test_staff_can_disable_version_global_version_manager(self):
        """test staff can disable version global version manager

        Returns:

        """
        self.fixture.global_template.is_current = False
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.disable_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id)
            in self.fixture.global_template.version_manager.disabled_versions
        )

    def test_staff_disable_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test staff disable version other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_superuser_can_disable_version_user_version_manager(self):
        """test superuser can disable version user version manager

        Returns:

        """
        self.fixture.user1_template.is_current = False
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id)
            in self.fixture.user1_template.version_manager.disabled_versions
        )

    def test_superuser_can_disable_version_global_version_manager(self):
        """test superuser can disable version global version manager

        Returns:

        """
        self.fixture.global_template.is_current = False
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id)
            in self.fixture.global_template.version_manager.disabled_versions
        )

    def test_superuser_can_disable_version_other_user_version_manager(self):
        """test superuser can disable version other user version manager

        Returns:

        """
        self.fixture.user2_template.is_current = False
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable_version(
            self.fixture.user2_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user2_template.id)
            in self.fixture.user2_template.version_manager.disabled_versions
        )


class TestVersionManagerSetCurrent(IntegrationBaseTestCase):
    """TestVersionManagerSetCurrent"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_set_current_user_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous set current user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user1_template, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_set_current_user_version_manager_with_access_right_raises_acl_error(
        self,
    ):
        """test anonymous set current user version manager with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user1_template, request=mock_request
            )

    def test_anonymous_set_current_global_version_manager_raises_access_control_error(
        self,
    ):
        """test anonymous set current global version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.global_template, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_anonymous_set_current_global_version_manager_with_access_right_raises_acl_error(
        self,
    ):
        """test anonymous set current global version manager with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.global_template, request=mock_request
            )

    def test_user_can_set_current_user_version_manager(self):
        """test user can set current user version manager

        Returns:

        """
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.set_current(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(
            str(self.fixture.user1_template.id),
            self.fixture.user1_template.version_manager.current,
        )

    def test_user_set_current_global_version_manager_raises_access_control_error(
        self,
    ):
        """test user set current global version manager raises access control error

        Returns:

        """
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.global_template, request=mock_request
            )

    def test_user_set_current_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test user set current other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user2_template, request=mock_request
            )

    def test_staff_can_set_current_user_version_manager(self):
        """test staff can set current user version manager

        Returns:

        """
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.set_current(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(
            str(self.fixture.user1_template.id),
            self.fixture.user1_template.version_manager.current,
        )

    def test_staff_can_set_current_global_version_manager(self):
        """test staff can set current global version manager

        Returns:

        """
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.set_current(
            self.fixture.global_template, request=mock_request
        )
        self.assertEqual(
            str(self.fixture.global_template.id),
            self.fixture.global_template.version_manager.current,
        )

    def test_staff_set_current_other_user_version_manager_raises_access_control_error(
        self,
    ):
        """test staff set current other user version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user2_template, request=mock_request
            )

    def test_superuser_can_set_current_user_version_manager(self):
        """test superuser can set current user version manager

        Returns:

        """
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.set_current(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(
            str(self.fixture.user1_template.id),
            self.fixture.user1_template.version_manager.current,
        )

    def test_superuser_can_set_current_global_version_manager(self):
        """test superuser can set current global version manager

        Returns:

        """
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.set_current(
            self.fixture.global_template, request=mock_request
        )
        self.assertEqual(
            str(self.fixture.global_template.id),
            self.fixture.global_template.version_manager.current,
        )

    def test_superuser_can_set_current_other_user_version_manager(self):
        """test superuser can set current other user version manager

        Returns:

        """
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.set_current(
            self.fixture.user2_template, request=mock_request
        )
        self.assertEqual(
            str(self.fixture.user2_template.id),
            self.fixture.user2_template.version_manager.current,
        )


class TestVersionManagerUpsert(IntegrationBaseTestCase):
    """TestVersionManagerUpsert"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_upsert_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test upsert user version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.user1_tvm, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_upsert_user_version_manager_as_anonymous_with_access_right_raises_acl_error(
        self,
    ):
        """test upsert user version manager as anonymous with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.user1_tvm, request=mock_request
            )

    def test_upsert_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test upsert global version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.global_tvm, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_upsert_global_version_manager_as_anonymous_with_access_right_raises_acl_error(
        self,
    ):
        """test upsert global version manager as anonymous with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.global_tvm, request=mock_request
            )

    def test_upsert_own_version_manager_as_user_saves(self):
        """test upsert own version manager as user saves

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.upsert(
            self.fixture.user1_tvm, request=mock_request
        )

    def test_upsert_other_users_version_manager_as_user_raises_access_control_error(
        self,
    ):
        """test upsert other users version manager as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.user2_tvm, request=mock_request
            )

    def test_upsert_global_version_manager_as_user_raises_access_control_error(
        self,
    ):
        """test upsert global version manager as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.global_tvm, request=mock_request
            )

    def test_upsert_own_version_manager_as_staff_saves(self):
        """test upsert own version manager as staff saves

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.upsert(
            self.fixture.user1_tvm, request=mock_request
        )

    def test_upsert_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        """test upsert other users version manager as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(
                self.fixture.user2_tvm, request=mock_request
            )

    def test_upsert_global_version_manager_as_staff_saves(self):
        """test upsert global version manager as staff saves

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.upsert(
            self.fixture.global_tvm, request=mock_request
        )

    def test_upsert_own_version_manager_as_superuser_saves(self):
        """test upsert own version manager as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.upsert(
            self.fixture.user1_tvm, request=mock_request
        )

    def test_upsert_other_users_version_manager_as_superuser_saves(self):
        """test upsert other users version manager as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.upsert(
            self.fixture.user2_tvm, request=mock_request
        )

    def test_upsert_global_version_manager_as_superuser_saves(self):
        """test upsert global version manager as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.upsert(
            self.fixture.global_tvm, request=mock_request
        )


class TestVersionManagerGetVersionNumber(IntegrationBaseTestCase):
    """TestVersionManagerGetVersionNumber"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_version_number_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get version number user version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user1_tvm,
                self.fixture.user1_template,
                request=mock_request,
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_version_number_user_version_manager_as_anonymous_with_access_right_raises_acl_error(
        self,
    ):
        """test get version number user version manager as anonymous with access right raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user1_tvm,
                self.fixture.user1_template,
                request=mock_request,
            )

    def test_get_version_number_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get version number global version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_version_number_global_version_manager_as_anonymous_with_access_right_returns_version_number(
        self,
    ):
        """test get version number global version manager as anonymous with access right returns version number

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        version_number = version_manager_api.get_version_number(
            self.fixture.global_tvm,
            self.fixture.global_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)

    def test_get_version_number_own_version_manager_as_user_returns_version_number(
        self,
    ):
        """test get version number own version manager as user returns version number

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_number = version_manager_api.get_version_number(
            self.fixture.user1_tvm,
            self.fixture.user1_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)

    def test_global_version_manager_as_user_returns_version_number(self):
        """test global version manager as user returns version number

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_number = version_manager_api.get_version_number(
            self.fixture.global_tvm,
            self.fixture.global_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)

    def test_get_version_number_other_users_version_manager_raises_access_control_error(
        self,
    ):
        """test get version number other users version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )

    def test_get_version_number_any_version_manager_as_superuser_returns_version_number(
        self,
    ):
        """test get version number any version manager as superuser returns version number

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_number = version_manager_api.get_version_number(
            self.fixture.user1_tvm,
            self.fixture.user1_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)
        version_number = version_manager_api.get_version_number(
            self.fixture.user2_tvm,
            self.fixture.user2_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)
        version_number = version_manager_api.get_version_number(
            self.fixture.global_tvm,
            self.fixture.global_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)

    def test_get_version_number_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        """test get version number other users version manager as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )


class TestVersionManagerGetVersionByNumber(IntegrationBaseTestCase):
    """TestVersionManagerGetVersionByNumber"""

    fixture = fixture_template_vm

    def setUp(self):
        """setup

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_version_by_number_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get version by number user version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user1_tvm, 1, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_version_by_number_user_version_manager_as_anonymous_with_access_right_raises_acl_error(
        self,
    ):
        """test get version by number user version manager as anonymous with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user1_tvm, 1, request=mock_request
            )

    def test_get_version_by_number_global_version_manager_as_anonymous_raises_acl_error(
        self,
    ):
        """test get version by number global version manager as anonymous raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.global_tvm, 1, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_version_by_number_global_version_manager_as_anonymous_with_access_right_returns_version(
        self,
    ):
        """test get version by number global version manager as anonymous with access right returns version

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        version = version_manager_api.get_version_by_number(
            self.fixture.global_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.global_template.id))

    def test_get_version_by_number_own_version_manager_as_user_returns_version(
        self,
    ):
        """test get version by number own version manager as user returns version

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version = version_manager_api.get_version_by_number(
            self.fixture.user1_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.user1_template.id))

    def test_global_version_manager_as_user_returns_version(self):
        """test global version manager as user returns version

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version = version_manager_api.get_version_by_number(
            self.fixture.global_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.global_template.id))

    def test_get_version_by_number_other_users_version_manager_raises_access_control_error(
        self,
    ):
        """test get version by number other users version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user2_tvm, 1, request=mock_request
            )

    def test_get_version_by_number_any_version_manager_as_superuser_returns_version(
        self,
    ):
        """test get version by number any version manager as superuser returns version

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version = version_manager_api.get_version_by_number(
            self.fixture.user1_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.user1_template.id))
        version = version_manager_api.get_version_by_number(
            self.fixture.user2_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.user2_template.id))
        version = version_manager_api.get_version_by_number(
            self.fixture.global_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.global_template.id))

    def test_get_version_by_number_other_users_version_manager_as_staff_raises_acl_error(
        self,
    ):
        """test get version by number other users version manager as staff raises acl error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user2_tvm, 1, request=mock_request
            )


class TestAccessControlCanRead(IntegrationBaseTestCase):
    """Test Access Control Can Read"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="2")
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    def test_access_control_can_read_as_anonymous_raises_access_control_error(
        self, mock_clean
    ):
        """test_access_control_can_read_as_anonymous_raises_access_control_error

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.user1_tvm
        mock_request = create_mock_request(user=self.anonymous_user)
        document_id = self.fixture.user1_tvm.id

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.can_read(
                self.fixture.user1_tvm.clean, document_id, mock_request
            )

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=False)
    def test_access_control_can_read_global_without_public_access_as_anonymous_raises_access_control_error(
        self, mock_clean
    ):
        """test_access_control_can_read_global_with_no_public_access_as_anonymous_raises_access_control_error

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.global_tvm
        mock_request = create_mock_request(user=self.anonymous_user)
        document_id = self.fixture.global_tvm.id

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.can_read(
                self.fixture.global_tvm.clean, document_id, mock_request
            )

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_access_control_can_read_global_with_public_access_as_anonymous_returns_function(
        self, mock_clean
    ):
        """test_access_control_can_read_global_with_public_access_as_anonymous_returns_function

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.global_tvm
        mock_request = create_mock_request(user=self.anonymous_user)
        document_id = self.fixture.global_tvm.id

        # Act
        result = access_control_api.can_read(
            self.fixture.global_tvm.clean, document_id, mock_request
        )

        # Assert
        self.assertEqual(result, self.fixture.global_tvm)

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    def test_access_control_can_read_as_user_raises_access_control_error(
        self, mock_clean
    ):
        """test_access_control_can_read_as_user_raises_access_control_error

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.user1_tvm
        mock_request = create_mock_request(user=self.user2)
        document_id = self.fixture.user1_tvm.id

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.can_read(
                self.fixture.user1_tvm.clean, document_id, mock_request
            )

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    def test_access_control_can_read_as_owner_returns_function(
        self, mock_clean
    ):
        """test_access_control_can_read_as_owner_returns_function

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.user1_tvm
        mock_request = create_mock_request(user=self.user1)
        document_id = self.fixture.user1_tvm.id

        # Act
        result = access_control_api.can_read(
            self.fixture.user1_tvm.clean, document_id, mock_request
        )

        # Assert
        self.assertEqual(result, self.fixture.user1_tvm)

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    def test_access_control_can_read_as_superuser_returns_function(
        self, mock_clean
    ):
        """test_access_control_can_read_as_superuser_returns_function

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.user1_tvm
        mock_request = create_mock_request(user=self.superuser1)
        document_id = self.fixture.user1_tvm.id

        # Act
        result = access_control_api.can_read(
            self.fixture.user1_tvm.clean, document_id, mock_request
        )

        # Assert
        self.assertEqual(result, self.fixture.user1_tvm)


class TestAccessControlCanReadGlobal(IntegrationBaseTestCase):
    """Test Access Control Can Read Global"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.fixture.insert_data()

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=False)
    def test_access_control_can_read_global_as_anonymous_without_public_access_raises_access_control_error(
        self,
    ):
        """test_access_control_can_read_global_as_anonymous_without_public_access_raises_access_control_error

        Returns:

        """
        # Arrange
        mock_request = create_mock_request(user=self.anonymous_user)

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.can_read_global(
                self.fixture.user1_tvm.clean, request=mock_request
            )

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_access_control_can_read_global_with_public_access_as_anonymous_returns_function(
        self, mock_clean
    ):
        """test_access_control_can_read_global_with_public_access_as_anonymous_returns_function

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.global_tvm
        mock_request = create_mock_request(user=self.anonymous_user)

        # Act
        result = access_control_api.can_read_global(
            self.fixture.global_tvm.clean, request=mock_request
        )

        # Assert
        self.assertEqual(result, self.fixture.global_tvm)

    @patch(
        "core_main_app.components.version_manager.models.VersionManager.clean"
    )
    def test_access_control_can_read_as_user_raises_access_control_error(
        self, mock_clean
    ):
        """test_access_control_can_read_as_user_raises_access_control_error

        Returns:

        """
        # Arrange
        mock_clean.return_value = self.fixture.global_tvm
        mock_request = create_mock_request(user=self.user1)

        # Act
        result = access_control_api.can_read_global(
            self.fixture.global_tvm.clean, request=mock_request
        )

        # Assert
        self.assertEqual(result, self.fixture.global_tvm)



""" Custom admin site for the XslTransformation model
"""
from django.contrib import admin


class CustomXslTransformationAdmin(admin.ModelAdmin):
    """CustomXslTransformationAdmin"""

    readonly_fields = ["checksum", "file"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding XslTransformation"""
        return False



""" Set of functions to define the rules for access control
"""
import logging

from django.conf import settings

from core_main_app.access_control.api import (
    has_perm_publish,
    can_write_in_workspace,
    check_can_write,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.data.access_control import (
    _get_read_accessible_workspaces_by_user,
)
from core_main_app.permissions import rights

logger = logging.getLogger(__name__)


def can_write_blob(func, blob, user):
    """Does the user has permission to write blob.

    Args:
        func:
        blob:
        user:

    Returns:

    """
    if user.is_anonymous:
        raise AccessControlError("Unable to insert blob if not authenticated.")

    return func(blob, user)


def has_perm_publish_blob(user):
    """Does the user have the permission to publish a blob.

    Args:
        user

    Returns
    """
    has_perm_publish(user, rights.PUBLISH_BLOB)


def can_write_blob_workspace(func, data, workspace, user):
    """Can user write data in workspace.

    Args:
        func:
        data:
        workspace:
        user:

    Returns:

    """
    return can_write_in_workspace(
        func, data, workspace, user, rights.PUBLISH_BLOB
    )


def can_write_metadata(func, blob, metadata, user):
    """Can user write metadata

    Args:
        func:
        blob:
        metadata:
        user:

    Returns:

    """
    check_can_write(blob, user)
    check_can_write(metadata, user)

    return func(blob, metadata, user)


def can_write_metadata_list(func, blob, metadata_list, user):
    """Can user write metadata list

    Args:
        func:
        blob:
        metadata_list:
        user:

    Returns:

    """
    check_can_write(blob, user)
    for metadata in metadata_list:
        check_can_write(metadata, user)

    return func(blob, metadata_list, user)


def filter_accessible_metadata(metadata, user):
    """Filter metadata to only return those that the user can read

    Args:
        metadata:
        user:

    Returns:

    """
    # If anon and not allowed to read data, return empty list
    if user is None or (
        user.is_anonymous and not settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT
    ):
        return []

    # If superuser, return unfiltered data
    if user.is_superuser:
        return metadata

    # Get list of workspace with read access
    accessible_workspaces = _get_read_accessible_workspaces_by_user(user)
    # Init list of metadata
    accessible_metadata = list()

    # Iterate through list of metadata
    for m in metadata:
        # If user is owner or can read metadata
        if str(m.user_id) == str(user.id) or (
            m.workspace is not None and m.workspace.id in accessible_workspaces
        ):
            # Add metadata to list
            accessible_metadata.append(m)

    return accessible_metadata



"""
Workspace model
"""
from django.core.exceptions import ObjectDoesNotExist
from django.core.validators import RegexValidator
from django.db import models
from django.db.models import Q

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES


class Workspace(models.Model):
    """
    Workspace class.
    """

    title = models.CharField(
        blank=False,
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Title must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    owner = models.CharField(blank=True, max_length=200, null=True)
    read_perm_id = models.CharField(blank=False, max_length=200)
    write_perm_id = models.CharField(blank=False, max_length=200)
    is_public = models.BooleanField(default=False)

    def clean(self):
        """

        Returns:
        """
        # Check the title
        if (
            self.owner is not None
            and self.title.lower() == "global public workspace"
        ):
            raise exceptions.ModelError(
                "You can't create a workspace with the title: " + self.title
            )

    @staticmethod
    def get_all():
        """Get all workspaces.

        Returns:

        """
        return Workspace.objects.all()

    @staticmethod
    def get_all_by_owner(user_id):
        """Get all workspaces created by the given user id.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects.filter(owner=str(user_id)).all()

    @staticmethod
    def get_by_id(workspace_id):
        """Return the workspace with the given id.

        Args:
            workspace_id

        Returns:
            Workspace (obj): Workspace object with the given id

        """
        try:
            return Workspace.objects.get(pk=workspace_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all_workspaces_with_read_access_by_user_id(
        user_id, read_permissions
    ):
        """Get all workspaces with read access for the given user id.

        Args:
            user_id
            read_permissions

        Returns:

        """
        return Workspace.objects.filter(
            Q(owner=str(user_id))
            | Q(read_perm_id__in=read_permissions)
            | Q(is_public=True)
        ).all()

    @staticmethod
    def get_all_workspaces_with_write_access_by_user_id(
        user_id, write_permissions
    ):
        """Get all workspaces with write access for the given user id.

        Args:
            user_id
            write_permissions

        Returns:

        """
        return Workspace.objects.filter(
            Q(owner=str(user_id)) | Q(write_perm_id__in=write_permissions)
        ).all()

    @staticmethod
    def get_all_workspaces_with_read_access_not_owned_by_user_id(
        user_id, read_permissions
    ):
        """Get all workspaces with read access not owned by the given user id.

        Args:
            user_id
            read_permissions

        Returns:

        """

        return Workspace.objects.filter(
            Q(read_perm_id__in=read_permissions) | Q(is_public=True),
            owner__ne=str(user_id),
        ).all()

    @staticmethod
    def get_all_workspaces_with_write_access_not_owned_by_user_id(
        user_id, write_permissions
    ):
        """Get all workspaces with write access not owned by the given user id.

        Args:
            user_id
            write_permissions

        Returns:

        """
        return Workspace.objects.filter(
            owner__ne=str(user_id), write_perm_id__in=write_permissions
        ).all()

    @staticmethod
    def get_all_public_workspaces():
        """Get all public workspaces.

        Args:

        Returns:

        """
        return Workspace.objects.filter(is_public=True).all()

    @staticmethod
    def get_all_other_public_workspaces(user_id):
        """Get all other public workspaces.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects.filter(
            owner__ne=str(user_id), is_public=True
        ).all()

    @staticmethod
    def get_non_public_workspace_owned_by_user_id(user_id):
        """Get the non public workspaces owned by the given user id.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects.filter(
            owner=str(user_id), is_public=False
        ).all()

    @staticmethod
    def get_public_workspaces_owned_by_user_id(user_id):
        """Get the public workspaces owned the given user id.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects.filter(
            owner=str(user_id), is_public=True
        ).all()

    @staticmethod
    def get_global_workspace():
        """Get global workspace.

        Returns:
        """
        try:
            return Workspace.objects.get(owner=None, is_public=True)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @property
    def is_global(self):
        """Get is global.

        Returns:
        """
        return self.is_public and self.owner is None

    def __str__(self):
        """Workspace object as string

        Returns:

        """
        return self.title



"""Results paginator util
"""
from django.core.paginator import PageNotAnInteger, EmptyPage, Paginator

from core_main_app.settings import RESULTS_PER_PAGE


class ResultsPaginator:
    """Results Paginator"""

    @staticmethod
    def get_results(results_list, page, results_per_page=RESULTS_PER_PAGE):
        """get_results

        Args:
            results_list:
            page:
            results_per_page:

        Returns:
        """
        # Pagination
        paginator = Paginator(results_list, results_per_page)

        try:
            results = paginator.page(int(page))
        except PageNotAnInteger:
            results = paginator.page(1)
        except EmptyPage:
            results = paginator.page(paginator.num_pages)

        return results



"""Version Manager unit tests
"""
from unittest.case import TestCase
from unittest.mock import Mock, patch, PropertyMock

from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request


class TestVersionManagerDisableVersion(TestCase):
    """TestVersionManagerDisableVersion"""

    def test_version_manager_disable_version_raises_exception_if_object_does_not_exist(
        self,
    ):
        """test version manager disable version raises exception if object does not exist

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_absent = _create_mock_object()

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.disable_version(
                mock_absent, request=mock_request
            )

    def test_version_manager_disable_current_version_throws_exception(self):
        """test version manager disable current version throws exception

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager = _create_mock_version_manager()
        mock_current = _create_mock_object()
        mock_version_manager.versions.append(str(mock_current.id))
        mock_version_manager.current = str(mock_current.id)

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.disable_version(
                mock_current, request=mock_request
            )

    @patch.object(
        TemplateVersionManager, "version_set", new_callable=PropertyMock
    )
    def test_version_manager_disable_version_raises_exception_if_new_current_does_not_exist(
        self, mock_versions
    ):
        """test version manager disable version raises exception if new current does not exist

        Args:
            mock_versions:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager = _create_version_manager()
        mock_absent = _create_mock_object()
        mock_current = _get_template()
        mock_current.is_current = True
        mock_current.version_manager = mock_version_manager
        mock_versions.return_value = [mock_current]

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.disable_version(
                mock_current, request=mock_request, new_current=mock_absent
            )


class TestVersionManagerSetCurrent(TestCase):
    """TestVersionManagerSetCurrent"""

    def test_version_manager_set_current_raises_api_error_if_disabled(self):
        """test version manager set current raises api error if disabled

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        to_disable = _create_mock_object()

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.set_current(to_disable, request=mock_request)


class TestVersionManagerGetActiveGlobalVersionManagerByTitle(TestCase):
    """TestVersionManagerGetActiveGlobalVersionManagerByTitle"""

    @patch.object(
        TemplateVersionManager, "get_active_global_version_manager_by_title"
    )
    def test_version_manager_get_returns_version_manager(
        self, mock_get_active_global
    ):
        """test version manager get returns version manager

        Args:
            mock_get_active_global:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        title = "Schema"
        mock_version_manager = _create_mock_version_manager(title=title)

        mock_get_active_global.return_value = mock_version_manager

        # Act
        result = template_version_manager_api.get_active_global_version_manager_by_title(
            title, request=mock_request
        )

        # Assert
        self.assertIsInstance(result, VersionManager)

    @patch.object(
        TemplateVersionManager, "get_active_global_version_manager_by_title"
    )
    def test_version_manager_get_raises_exception_if_object_does_not_exist(
        self, mock_get_active_global
    ):
        """test version manager get raises exception if object does not exist

        Args:
            mock_get_active_global:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_absent_title = "Schema"
        mock_get_active_global.side_effect = DoesNotExist("")

        # Act + Assert
        with self.assertRaises(DoesNotExist):
            template_version_manager_api.get_active_global_version_manager_by_title(
                mock_absent_title, request=mock_request
            )


class TestVersionManagerGetVersionByNumber(TestCase):
    """TestVersionManagerGetVersionByNumber"""

    @patch.object(
        TemplateVersionManager, "version_set", new_callable=PropertyMock
    )
    def test_version_manager_get_returns_version(self, mock_versions):
        """test version manager get returns version

        Args:
            mock_versions:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version = _get_template()
        mock_versions.return_value = [version]
        version_manager = _create_version_manager("Schema1", [version])

        # Act
        result = version_manager_api.get_version_by_number(
            version_manager, 1, request=mock_request
        )

        # Assert
        self.assertEqual(result, str(version.id))

    def test_version_manager_get_raises_exception_if_object_does_not_exist(
        self,
    ):
        """test version manager get raises exception if object does not exist

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version_manager = _create_version_manager("Schema1", [_get_template()])

        # Act + Assert
        with self.assertRaises(IndexError):
            version_manager_api.get_version_by_number(
                version_manager, 2, request=mock_request
            )


class TestVersionManagerGetVersionNumber(TestCase):
    """TestVersionManagerGetVersionNumber"""

    @patch.object(
        TemplateVersionManager, "version_set", new_callable=PropertyMock
    )
    def test_version_manager_get_returns_number(self, mock_versions):
        """test version manager get returns number

        Args:
            mock_versions:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version = _get_template()
        mock_versions.return_value = [version]
        version_manager = _create_version_manager("Schema1", [version])

        # Act
        result = version_manager_api.get_version_number(
            version_manager, version.id, request=mock_request
        )

        # Assert
        self.assertEqual(result, 1)

    def test_version_manager_get_raises_exception_if_object_does_not_exist(
        self,
    ):
        """test version manager get raises exception if object does not exist

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        # version = 1
        absent_version = -1
        version_manager = _create_version_manager("Schema1", [_get_template()])

        # Act + Assert
        with self.assertRaises(ValueError):
            version_manager_api.get_version_number(
                version_manager, absent_version, request=mock_request
            )


def _create_mock_version_manager(title=""):
    """Create a mock version manager

    Args:
        title:

    Returns:

    """
    mock_vm = Mock(spec=VersionManager)
    mock_vm.title = title
    mock_vm.current = 1
    mock_vm.versions = [str(mock_vm.current)]
    mock_vm.id = 1
    return mock_vm


def _create_mock_object():
    """Create a mock object

    Returns:

    """
    mock_object = Mock()
    mock_object.id = 1
    return mock_object


def _create_version_manager(title="", versions=None):
    """Return a templates version manager

    Args:
        title:
        versions:

    Returns:

    """
    tvm = TemplateVersionManager(
        id=1,
        title=title,
    )
    if versions:
        for template in versions:
            template.version_manager = tvm

    return tvm


def _get_template():
    """Get template

    Returns:

    """
    template = Template()
    template.id = 1
    xsd = (
        "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
        "<xs:element name='tag'></xs:element></xs:schema>"
    )
    template.content = xsd
    return template



""" Integration Test for Blob Rest API
"""
from os.path import join, dirname, abspath
from unittest.mock import patch

from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import override_settings
from rest_framework import status
from tests.components.blob.fixtures.fixtures import (
    BlobFixtures,
    AccessControlBlobFixture,
)
from tests.components.data.fixtures.fixtures import (
    AccessControlBlobWithMetadataFixture,
)

from core_main_app.components.blob.models import Blob
from core_main_app.components.workspace.models import Workspace
from core_main_app.rest.blob import views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock

RESOURCES_PATH = join(dirname(abspath(__file__)), "data")
fixture_blob = BlobFixtures()
fixture_blob_workspace = AccessControlBlobFixture()
fixture_blob_metadata = AccessControlBlobWithMetadataFixture()


class TestBlobListAdmin(IntegrationBaseTestCase):
    """TestBlobListAdmin"""

    fixture = fixture_blob

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_user_returns_http_403(self):
        """test_get_as_user_returns_http_403

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobListAdmin.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_superuser_returns_all_blobs(self):
        """test_get_as_superuser_returns_all_blobs

        Returns:

        """
        # Arrange
        user = create_mock_user("1", is_staff=True, is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobListAdmin.as_view(), user
        )

        # Assert
        self.assertEqual(len(response.data), 3)


class TestBlobList(IntegrationBaseTestCase):
    """TestBlobList"""

    fixture = fixture_blob

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = {
            "blob": SimpleUploadedFile("blob.txt", b"blob"),
        }

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        """test_get_returns_http_200

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(), user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_all_user_blobs(self):
        """test_get_returns_all_user_blobs

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_superuser_returns_all_user_blobs(self):
        """test_get_as_superuser_returns_all_user_blobs

        Returns:

        """
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_correct_name_returns_http_200(self):
        """test_get_filtered_by_correct_name_returns_http_200

        Returns:

        """
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(),
            user,
            data={"filename": self.fixture.blob_1.filename},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_correct_name_returns_correct_blob(self):
        """test_get_filtered_by_correct_name_returns_correct_blob

        Returns:

        """
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(),
            user,
            data={"filename": self.fixture.blob_1.filename},
        )

        # Assert
        self.assertEqual(
            response.data[0]["filename"], self.fixture.blob_1.filename
        )

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_incorrect_name_returns_http_200(self):
        """test_get_filtered_by_incorrect_name_returns_http_200

        Returns:

        """
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(), user, data={"filename": "incorrect"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_incorrect_name_returns_empty_list(self):
        """test_get_filtered_by_incorrect_name_returns_empty_list

        Returns:

        """
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(), user, data={"filename": "incorrect"}
        )

        # Assert
        self.assertEqual(len(response.data), 0)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201(self):
        """test_post_returns_http_201

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data, content_type=None
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data["filename"], self.data["blob"].name)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_with_filename_returns_http_201(self):
        """test_post_returns_http_201

        Returns:

        """
        # Arrange
        user = create_mock_user("1")
        self.data["filename"] = "test.txt"

        # Act
        response = RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data, content_type=None
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data["filename"], self.data["filename"])

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_adds_an_entry_in_database(self):
        """test_post_adds_an_entry_in_database

        Returns:

        """
        # Arrange
        user = create_mock_user("1")
        number_of_blobs = Blob.get_all().count()

        # Act
        RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data, content_type=None
        )

        # Assert
        self.assertEqual(Blob.get_all().count(), number_of_blobs + 1)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_incorrect_file_parameter_returns_http_400(self):
        """test_post_incorrect_file_parameter_returns_http_400

        Returns:

        """
        # Arrange
        user = create_mock_user("1")
        self.data["blob"] = "test.txt"

        # Act
        response = RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data, content_type=None
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestBlobDetail(IntegrationBaseTestCase):
    """TestBlobDetail"""

    fixture = fixture_blob

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        """test_get_returns_http_200

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_blob(self):
        """test_get_returns_blob

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(
            response.data["filename"], self.fixture.blob_1.filename
        )

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_wrong_id_returns_http_404(self):
        """test_get_wrong_id_returns_http_404

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(), user, param={"pk": -1}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_other_user_blob_returns_http_403(self):
        """test_get_other_user_blob_returns_http_403

        Returns:

        """
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_returns_http_204(self):
        """test_delete_returns_http_204

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_deletes_one_blob_from_database(self):
        """test_delete_deletes_one_blob_from_database

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        RequestMock.do_request_delete(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(len(Blob.get_all()), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_wrong_id_returns_http_404(self):
        """test_delete_wrong_id_returns_http_404

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(), user, param={"pk": -1}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_other_user_blob_returns_http_403(self):
        """test_delete_other_user_blob_returns_http_403

        Returns:

        """
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_other_user_as_superuser_returns_http_204(self):
        """test_delete_other_user_as_superuser_returns_http_204

        Returns:

        """
        # Arrange
        user = create_mock_user("2", is_superuser=True)

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobDownload(IntegrationBaseTestCase):
    """TestBlobDownload"""

    fixture = fixture_blob

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.blob = open(join(RESOURCES_PATH, "test.txt"), "r").read()

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        """test_get_returns_http_200

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDownload.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_wrong_id_returns_http_404(self):
        """test_get_wrong_id_returns_http_404

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDownload.as_view(), user, param={"pk": -1}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_other_user_blob_returns_http_403(self):
        """test_get_other_user_blob_returns_http_403

        Returns:

        """
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDownload.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestBlobDeleteList(IntegrationBaseTestCase):
    """TestBlobDeleteList"""

    fixture = fixture_blob

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = [
            {"id": str(self.fixture.blob_1.id)},
            {"id": str(self.fixture.blob_2.id)},
        ]

    def test_post_a_list_containing_other_user_blob_returns_http_403(self):
        """test_post_a_list_containing_other_user_blob_returns_http_403

        Returns:

        """
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.BlobDeleteList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_post_a_list_containing_other_user_blob_as_superuser_returns_http_204(
        self,
    ):
        """test_post_a_list_containing_other_user_blob_as_superuser_returns_http_204

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.BlobDeleteList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobAssign(IntegrationBaseTestCase):
    """TestBlobAssign"""

    fixture = fixture_blob_workspace

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200(self, blob_get_by_id, workspace_get_by_id):
        """test_get_returns_http_200

        Args:
            blob_get_by_id:
            workspace_get_by_id:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_1

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_updates_workspace(
        self, blob_get_by_id, workspace_get_by_id
    ):
        """test_assign_blob_to_workspace_updates_workspace

        Args:
            blob_get_by_id:
            workspace_get_by_id:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(
            str(blob.workspace.id), str(self.fixture.workspace_2.id)
        )

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_returns_http_200(
        self, blob_get_by_id, workspace_get_by_id
    ):
        """test_assign_blob_to_workspace_returns_http_200

        Args:
            blob_get_by_id:
            workspace_get_by_id:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    def test_assign_bad_blob_id_returns_http_404(self, workspace_get_by_id):
        """test_assign_bad_blob_id_returns_http_404

        Args:
            workspace_get_by_id:

        Returns:

        """
        # Arrange
        fake_blob_id = -1
        user = create_mock_user("1", is_superuser=True)
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={
                "pk": fake_blob_id,
                "workspace_id": self.fixture.workspace_2.id,
            },
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Blob, "get_by_id")
    def test_assign_bad_workspace_id_returns_http_404(self, blob_get_by_id):
        """test_assign_bad_workspace_id_returns_http_404

        Args:
            blob_get_by_id:

        Returns:

        """
        # Arrange
        fake_workspace_id = -1
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": fake_workspace_id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestBlobChangeOwner(IntegrationBaseTestCase):
    """TestBlobChangeOwner"""

    fixture = fixture_blob_workspace

    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200_if_user_is_superuser(
        self, blob_get_by_id, user_get_by_id
    ):
        """test_get_returns_http_200_if_user_is_superuser

        Args:
            blob_get_by_id:
            user_get_by_id:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user_request = create_mock_user(
            "65467", is_staff=True, is_superuser=True
        )
        user_new_owner = create_mock_user("123")
        blob_get_by_id.return_value = blob
        user_get_by_id.return_value = user_new_owner

        # Act
        response = RequestMock.do_request_patch(
            views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": blob.id, "user_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200_if_user_is_not_superuser_but_have_access_to_the_blob(
        self, blob_get_by_id, user_get_by_id
    ):
        """test_get_returns_http_200_if_user_is_not_superuser_but_have_access_to_the_blob

        Args:
            blob_get_by_id:
            user_get_by_id:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user_request = create_mock_user("1", is_staff=True)
        user_new_owner = create_mock_user("123")
        blob_get_by_id.return_value = blob
        user_get_by_id.return_value = user_new_owner

        # Act
        response = RequestMock.do_request_patch(
            views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": blob.id, "user_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_403_if_user_is_not_superuser_or_have_no_access_to_the_blob(
        self, blob_get_by_id, user_get_by_id, read_access_mock
    ):
        """test_get_returns_http_403_if_user_is_not_superuser_or_have_no_access_to_the_blob

        Args:
            blob_get_by_id:
            user_get_by_id:
            read_access_mock:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user_request = create_mock_user("65467", is_staff=True)
        user_new_owner = create_mock_user("123")
        blob_get_by_id.return_value = blob
        user_get_by_id.return_value = user_new_owner
        read_access_mock.return_value = []

        # Act
        response = RequestMock.do_request_patch(
            views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": blob.id, "user_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestBlobAddMetadata(IntegrationBaseTestCase):
    """TestBlobAddMetadata"""

    fixture = fixture_blob_metadata

    def test_add_metadata_authorized_returns_http_200(
        self,
    ):
        """test_add_metadata_authorized_returns_http_200

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_1
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": self.fixture.data_4.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.blob.api.add_metadata")
    def test_add_metadata_with_errors_returns_http_500(self, add_metadata):
        """test_add_metadata_with_errors_returns_http_500

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_1
        mock_user = create_mock_user("1")
        add_metadata.side_effect = Exception()

        # Act
        response = RequestMock.do_request_post(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": self.fixture.data_4.id},
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_add_metadata_unauthorized_returns_http_403(
        self,
        get_all_workspaces_with_read_access_by_user,
        get_all_workspaces_with_write_access_by_user,
    ):
        """test_add_metadata_unauthorized_returns_http_403

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_3
        mock_user = create_mock_user("1")
        get_all_workspaces_with_read_access_by_user.return_value = []
        get_all_workspaces_with_write_access_by_user.return_value = []

        # Act
        response = RequestMock.do_request_post(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": self.fixture.data_4.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_add_metadata_wrong_blob_id_returns_http_404(
        self,
    ):
        """test_add_metadata_wrong_blob_id_returns_http_404

        Args:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": "1234", "metadata_id": self.fixture.data_4.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_add_metadata_wrong_data_id_returns_http_404(
        self,
    ):
        """test_add_metadata_wrong_data_id_returns_http_404

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_1
        mock_user = create_mock_user("1")
        # Act
        response = RequestMock.do_request_post(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": "1234"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestBlobRemoveMetadata(IntegrationBaseTestCase):
    """TestBlobRemoveMetadata"""

    fixture = fixture_blob_metadata

    def test_remove_metadata_authorized_returns_http_200(
        self,
    ):
        """test_remove_metadata_authorized_returns_http_200

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_1
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": self.fixture.data_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.blob.api.remove_metadata")
    def test_remove_metadata_with_errors_returns_http_500(
        self, remove_metadata
    ):
        """test_remove_metadata_with_errors_returns_http_500

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_1
        mock_user = create_mock_user("1")
        remove_metadata.side_effect = Exception()

        # Act
        response = RequestMock.do_request_delete(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": self.fixture.data_1.id},
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_remove_metadata_unauthorized_returns_http_403(
        self,
        get_all_workspaces_with_read_access_by_user,
        get_all_workspaces_with_write_access_by_user,
    ):
        """test_remove_metadata_unauthorized_returns_http_403

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_3
        mock_user = create_mock_user("1")
        get_all_workspaces_with_read_access_by_user.return_value = []
        get_all_workspaces_with_write_access_by_user.return_value = []

        # Act
        response = RequestMock.do_request_delete(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": self.fixture.data_32.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_remove_metadata_wrong_blob_id_returns_http_404(
        self,
    ):
        """test_remove_metadata_wrong_blob_id_returns_http_404

        Args:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": "1234", "metadata_id": self.fixture.data_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_remove_metadata_wrong_data_id_returns_http_404(
        self,
    ):
        """test_remove_metadata_wrong_data_id_returns_http_404

        Args:

        Returns:

        """
        # Arrange
        blob = self.fixture.blob_1
        mock_user = create_mock_user("1")
        # Act
        response = RequestMock.do_request_delete(
            views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": blob.id, "metadata_id": "1234"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



""" Access control testing for Blob.
"""
import unittest
from unittest.mock import patch

from django.core.files.uploadedfile import SimpleUploadedFile
from tests.components.blob.fixtures.fixtures import AccessControlBlobFixture
from tests.components.data.fixtures.fixtures import (
    AccessControlBlobWithMetadataFixture,
)

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.components.blob import api as blob_api
from core_main_app.components.blob.models import Blob
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user

fixture_blob = AccessControlBlobFixture()
fixture_blob_metadata = AccessControlBlobWithMetadataFixture()


class TestBlobGetById(IntegrationBaseTestCase):
    """TestBlobGetById"""

    fixture = fixture_blob

    def test_get_by_id_owner_with_read_access_returns_blob(self):
        """test_get_by_id_owner_with_read_access_returns_blob

        Returns:

        """
        blob_id = self.fixture.blob_collection[
            fixture_blob.USER_1_WORKSPACE_1
        ].id
        mock_user = _create_user("1")
        blob = blob_api.get_by_id(blob_id, mock_user)
        self.assertTrue(isinstance(blob, Blob))

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_by_id_user_without_read_access_raises_error(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_get_by_id_user_without_read_access_raises_error

        Args:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        blob_id = self.fixture.blob_collection[
            fixture_blob.USER_1_WORKSPACE_1
        ].id
        get_all_workspaces_with_read_access_by_user.return_value = []
        mock_user = _create_user("2")
        with self.assertRaises(AccessControlError):
            blob_api.get_by_id(blob_id, mock_user)

    def test_get_by_id_owner_no_workspace_read_access_returns_blob(self):
        """test_get_by_id_owner_no_workspace_read_access_returns_blob

        Returns:

        """
        blob_id = self.fixture.blob_collection[
            fixture_blob.USER_1_NO_WORKSPACE
        ].id
        mock_user = _create_user("1")
        blob = blob_api.get_by_id(blob_id, mock_user)
        self.assertTrue(isinstance(blob, Blob))

    def test_get_by_id_not_owner_no_workspace_raises_error(self):
        """test_get_by_id_not_owner_no_workspace_raises_error

        Returns:

        """
        blob_id = self.fixture.blob_collection[
            fixture_blob.USER_1_NO_WORKSPACE
        ].id
        mock_user = _create_user("2")
        with self.assertRaises(AccessControlError):
            blob_api.get_by_id(blob_id, mock_user)


class TestBlobGetAll(IntegrationBaseTestCase):
    """TestBlobGetAll"""

    fixture = fixture_blob

    def test_get_all_as_superuser_returns_all_blob(self):
        """test_get_all_as_superuser_returns_all_blob

        Returns:

        """
        mock_user = _create_user("1", is_superuser=True)
        data_list = blob_api.get_all(mock_user)
        self.assertTrue(len(data_list) == len(self.fixture.blob_collection))

    def test_get_all_as_user_raises_error(self):
        """test_get_all_as_user_raises_error

        Returns:

        """
        mock_user = _create_user("1")
        with self.assertRaises(AccessControlError):
            blob_api.get_all(mock_user)


class TestBlobGetAllByUser(IntegrationBaseTestCase):
    """TestBlobGetAllByUser"""

    fixture = fixture_blob

    def test_get_all_by_user_returns_owned_blob(self):
        """test_get_all_by_user_returns_owned_blob

        Returns:

        """
        mock_user = _create_user("1")
        blob_list = blob_api.get_all_by_user(mock_user)
        self.assertTrue(len(blob_list) == 2)
        self.assertTrue(blob.id == "1" for blob in blob_list)

    def test_get_all_by_user_returns_no_blob_if_owns_zero(self):
        """test_get_all_by_user_returns_no_blob_if_owns_zero

        Returns:

        """
        mock_user = _create_user("3")
        blob_list = blob_api.get_all_by_user(mock_user)
        self.assertTrue(len(blob_list) == 0)

    def test_get_all_by_user_as_superuser_returns_own_blob(self):
        """test_get_all_by_user_as_superuser_returns_own_blob

        Returns:

        """
        mock_user = _create_user("1", is_superuser=True)
        blob_list = blob_api.get_all_by_user(mock_user)
        self.assertTrue(len(blob_list) == 2)
        self.assertTrue(blob.user_id == "1" for blob in blob_list)


class TestBlobGetAllByWorkspace(IntegrationBaseTestCase):
    """TestBlobGetAllByWorkspace"""

    fixture = fixture_blob

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_all_by_workspace_returns_owned_blob(
        self,
        get_all_workspaces_with_write_access_by_user,
        get_all_workspaces_with_read_access_by_user,
    ):
        """test_get_all_by_workspace_returns_owned_blob

        Args:
            get_all_workspaces_with_write_access_by_user:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        mock_user = _create_user(self.fixture.USER_1_WORKSPACE_1)
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        blob_list = blob_api.get_all_by_workspace(
            self.fixture.workspace_1, mock_user
        )
        self.assertTrue(blob.user_id == mock_user.id for blob in blob_list)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_all_by_workspace_returns_no_blob_if_owns_zero(
        self,
        get_all_workspaces_with_write_access_by_user,
        get_all_workspaces_with_read_access_by_user,
    ):
        """test_get_all_by_workspace_returns_no_blob_if_owns_zero

        Args:
            get_all_workspaces_with_write_access_by_user:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        mock_user = _create_user("7")
        get_all_workspaces_with_write_access_by_user.return_value = []
        get_all_workspaces_with_read_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.get_all_by_workspace(self.fixture.workspace_1, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_all_by_workspace_as_superuser_returns_owned_blob(
        self,
        get_all_workspaces_with_write_access_by_user,
        get_all_workspaces_with_read_access_by_user,
    ):
        """test_get_all_by_workspace_as_superuser_returns_owned_blob

        Args:
            get_all_workspaces_with_write_access_by_user:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        blob_list = blob_api.get_all_by_workspace(
            self.fixture.workspace_1, mock_user
        )
        self.assertTrue(blob.user_id == mock_user.id for blob in blob_list)


class TestBlobDelete(IntegrationBaseTestCase):
    """TestBlobDelete"""

    fixture = fixture_blob

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_own_blob_in_accessible_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_delete_own_blob_in_accessible_workspace_deletes_blob

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_1
        ]
        blob_api.delete(
            fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1],
            mock_user,
        )

    # FIXME: test is not true. Deleting own data in workspace without write access raises ACL error.
    # (FIXME note also found in ACL code.)
    @unittest.skip("Test is not True.")
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_own_blob_in_not_accessible_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_delete_own_blob_in_not_accessible_workspace_deletes_blob

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        blob_api.delete(
            fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1],
            mock_user,
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_others_blob_in_accessible_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_delete_others_blob_in_accessible_workspace_deletes_blob

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_2
        ]
        blob_api.delete(
            fixture_blob.blob_collection[fixture_blob.USER_2_WORKSPACE_2],
            mock_user,
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_others_blob_not_accessible_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_delete_others_blob_not_accessible_workspace_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_1
        ]
        with self.assertRaises(AccessControlError):
            blob_api.delete(
                fixture_blob.blob_collection[fixture_blob.USER_2_WORKSPACE_2],
                mock_user,
            )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_own_blob_not_in_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_delete_own_blob_not_in_workspace_deletes_blob

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.delete(
                fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1],
                mock_user,
            )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_others_blob_not_in_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_delete_others_blob_not_in_workspace_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.delete(
                fixture_blob.blob_collection[fixture_blob.USER_2_NO_WORKSPACE],
                mock_user,
            )


class TestBlobChangeOwner(IntegrationBaseTestCase):
    """TestBlobChangeOwner"""

    fixture = fixture_blob

    def test_change_owner_from_owner_to_owner_ok(self):
        """test_change_owner_from_owner_to_owner_ok

        Returns:

        """
        mock_owner = _create_user("1")
        blob_api.change_owner(
            document=fixture_blob.blob_collection[
                fixture_blob.USER_1_NO_WORKSPACE
            ],
            new_user=mock_owner,
            user=mock_owner,
        )

    def test_change_owner_from_owner_to_user_ok(self):
        """test_change_owner_from_owner_to_user_ok

        Returns:

        """
        mock_owner = _create_user("1")
        mock_user = _create_user("2")
        blob_api.change_owner(
            document=fixture_blob.blob_collection[
                fixture_blob.USER_1_NO_WORKSPACE
            ],
            new_user=mock_user,
            user=mock_owner,
        )

    def test_change_owner_from_user_to_user_raises_exception(self):
        """test_change_owner_from_user_to_user_raises_exception

        Returns:

        """
        mock_owner = _create_user("1")
        mock_user = _create_user("2")
        with self.assertRaises(AccessControlError):
            blob_api.change_owner(
                document=fixture_blob.blob_collection[
                    fixture_blob.USER_1_NO_WORKSPACE
                ],
                new_user=mock_owner,
                user=mock_user,
            )

    def test_change_owner_as_superuser_ok(self):
        """test_change_owner_as_superuser_ok

        Returns:

        """
        mock_user = _create_user("2", is_superuser=True)
        blob_api.change_owner(
            document=fixture_blob.blob_collection[
                fixture_blob.USER_1_NO_WORKSPACE
            ],
            new_user=mock_user,
            user=mock_user,
        )


class TestBlobInsert(IntegrationBaseTestCase):
    """TestBlobInsert"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user = _create_user("1")
        self.superuser = _create_user("2", True)
        self.blob = Blob(
            filename="blob",
            user_id="1",
            blob=SimpleUploadedFile("blob.txt", b"blob"),
        )

    def test_insert_blob_as_anonymous_raises_error(self):
        """test_insert_blob_as_anonymous_raises_error

        Returns:

        """
        with self.assertRaises(AccessControlError):
            blob_api.insert(self.blob, self.anonymous_user)

    def test_insert_blob_as_user_creates_blob(
        self,
    ):
        """test_insert_blob_as_user_creates_blob

        Returns:

        """
        blob_api.insert(self.blob, self.user)

    def test_insert_blob_as_superuser_creates_blob(
        self,
    ):
        """test_insert_blob_as_superuser_creates_blob

        Returns:

        """
        blob_api.insert(self.blob, self.superuser)

    def test_edit_blob_as_user_raises_error(
        self,
    ):
        """test_edit_blob_as_user_raises_error

        Returns:

        """
        with self.assertRaises(exceptions.ApiError):
            blob_api.insert(
                fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1],
                self.user,
            )

    def test_edit_blob_as_superuser_raises_error(
        self,
    ):
        """test_edit_blob_as_superuser_raises_error

        Returns:

        """
        with self.assertRaises(exceptions.ApiError):
            blob_api.insert(
                fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
                self.superuser,
            )


class TestBlobAssign(IntegrationBaseTestCase):
    """TestBlobAssign"""

    fixture = fixture_blob

    def test_assign_blob_as_anonymous_raises_error(self):
        """test_assign_blob_as_anonymous_raises_error

        Returns:

        """
        anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        with self.assertRaises(AccessControlError):
            blob_api.assign(
                self.fixture.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
                fixture_blob.workspace_1,
                anonymous_user,
            )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_assign_own_blob_to_accessible_workspace_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_assign_own_blob_to_accessible_workspace_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_1
        ]
        blob_api.assign(
            fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
            fixture_blob.workspace_1,
            user,
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_assign_own_blob_to_inaccessible_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_assign_own_blob_to_inaccessible_workspace_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.assign(
                fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1],
                fixture_blob.workspace_2,
                user,
            )

    def test_assign_own_blob_with_no_workspace_to_none_ok(self):
        """test_assign_own_blob_with_no_workspace_to_none_ok

        Returns:

        """
        user = _create_user("1")
        blob_api.assign(
            fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
            None,
            user,
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_assign_others_blob_to_accessible_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_assign_others_blob_to_accessible_workspace_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_1
        ]
        with self.assertRaises(AccessControlError):
            blob_api.assign(
                fixture_blob.blob_collection[fixture_blob.USER_2_WORKSPACE_2],
                fixture_blob.workspace_1,
                user,
            )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_assign_others_blob_to_inaccessible_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_assign_others_blob_to_inaccessible_workspace_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.assign(
                fixture_blob.blob_collection[fixture_blob.USER_2_WORKSPACE_2],
                fixture_blob.workspace_1,
                user,
            )

    def test_assign_blob_as_superuser_ok(
        self,
    ):
        """test_assign_blob_as_superuser_ok

        Returns:

        """
        user = _create_user("1", True)
        blob_api.assign(
            fixture_blob.blob_collection[fixture_blob.USER_2_NO_WORKSPACE],
            fixture_blob.workspace_1,
            user,
        )


class TestBlobMetadata(IntegrationBaseTestCase):
    """TestBlobMetadata"""

    fixture = fixture_blob_metadata

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_blob_metadata_as_owner_returns_metadata(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_blob_metadata_as_owner_returns_metadata

        Args:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        mock_user = create_mock_user(1)
        get_all_workspaces_with_read_access_by_user.return_value = []
        metadata = blob.metadata(mock_user)
        self.assertEqual(metadata, [self.fixture.data_1])

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_blob_metadata_private_metadata_as_other_user_returns_nothing(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_blob_metadata_private_metadata_as_other_user_raises_acl_error

        Args:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        mock_user = create_mock_user(2)
        get_all_workspaces_with_read_access_by_user.return_value = []
        metadata = blob.metadata(mock_user)
        self.assertTrue(len(metadata) == 0)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_blob_metadata_as_other_user_without_access_returns_nothing(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_blob_metadata_private_metadata_as_other_user_raises_acl_error

        Args:
            get_all_workspaces_with_read_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_read_access_by_user.return_value = []
        metadata = blob.metadata(mock_user)
        self.assertTrue(len(metadata) == 0)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_blob_metadata_other_user_metadata_in_workspace_with_access_returns_metadata(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_blob_metadata_other_user_metadata_in_workspace_with_access_returns_metadata

        Returns:

        """
        blob = self.fixture.blob_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        metadata = blob.metadata(mock_user)
        self.assertEqual(metadata, [self.fixture.data_2])

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_blob_metadata_other_user_metadata_in_workspace_without_access_does_not_return_metadata(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_blob_metadata_other_user_metadata_in_workspace_without_access_does_not_return_metadata

        Returns:

        """
        blob = self.fixture.blob_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_read_access_by_user.return_value = []
        metadata = blob.metadata(mock_user)
        self.assertTrue(len(metadata) == 0)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_blob_metadata_other_user_metadata_returns_only_accessible_metadata(
        self, get_all_workspaces_with_read_access_by_user
    ):
        """test_blob_metadata_other_user_metadata_returns_only_accessible_metadata

        Returns:

        """
        blob = self.fixture.blob_3
        mock_user = create_mock_user(1)
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        metadata = blob.metadata(mock_user)
        self.assertEqual(metadata, [self.fixture.data_31])


class TestAddMetadata(IntegrationBaseTestCase):
    """TestAddMetadata"""

    fixture = fixture_blob_metadata

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_cannot_write_blob_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_cannot_write_blob_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.add_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_cannot_write_data_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_cannot_write_data_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(2)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.add_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_own_blob_and_own_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_own_blob_and_own_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        blob_api.add_metadata(blob, data, mock_user)
        self.assertEqual(data._blob, blob)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_own_blob_and_accessible_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_own_blob_and_accessible_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.add_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_can_write_blob_and_own_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_can_write_blob_and_own_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]

        blob_api.add_metadata(blob, data, mock_user)
        self.assertEqual(data._blob, blob)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_can_write_blob_and_can_write_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_can_write_blob_and_can_write_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]

        blob_api.add_metadata(blob, data, mock_user)
        self.assertEqual(data._blob, blob)


class TestAddMetadataList(IntegrationBaseTestCase):
    """TestAddMetadataList"""

    fixture = fixture_blob_metadata

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_list_cannot_write_blob_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_list_cannot_write_blob_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.add_metadata_list(blob, [data], mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_list_cannot_write_data_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_list_cannot_write_data_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(2)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.add_metadata_list(blob, [data], mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_list_own_blob_and_own_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_list_own_blob_and_own_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        blob_api.add_metadata_list(blob, [data], mock_user)
        self.assertEqual(data._blob, blob)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_list_own_blob_and_accessible_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_list_own_blob_and_accessible_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.add_metadata_list(blob, [data], mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_list_can_write_blob_and_own_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_list_can_write_blob_and_own_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]

        blob_api.add_metadata_list(blob, [data], mock_user)
        self.assertEqual(data._blob, blob)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_add_metadata_list_can_write_blob_and_can_write_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_add_metadata_list_can_write_blob_and_can_write_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]

        blob_api.add_metadata_list(blob, [data], mock_user)
        self.assertEqual(data._blob, blob)


class TestRemoveMetadata(IntegrationBaseTestCase):
    """TestRemoveMetadata"""

    fixture = fixture_blob_metadata

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_not_linked_to_blob_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_not_linked_to_blob_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_4
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(Exception):
            blob_api.remove_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_cannot_write_blob_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_cannot_write_blob_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.remove_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_cannot_write_data_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_cannot_write_data_raises_error

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_2
        data = self.fixture.data_4
        mock_user = create_mock_user(2)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.remove_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_own_blob_and_own_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_own_blob_and_own_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_1
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        blob_api.remove_metadata(blob, data, mock_user)
        self.assertEqual(blob._metadata.count(), 0)
        self.assertEqual(data._blob, None)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_own_blob_and_accessible_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_own_blob_and_accessible_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_1
        data = self.fixture.data_2
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = []

        with self.assertRaises(AccessControlError):
            blob_api.remove_metadata(blob, data, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_can_write_blob_and_can_write_data_ok(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_can_write_blob_and_can_write_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_3
        data = self.fixture.data_31
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]

        blob_api.remove_metadata(blob, data, mock_user)
        self.assertEqual(blob._metadata.count(), 1)
        self.assertEqual(data._blob, None)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_remove_metadata_can_write_blob_and_cannot_write_data_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        """test_remove_metadata_can_write_blob_and_can_write_data_ok

        Args:
            get_all_workspaces_with_write_access_by_user:

        Returns:

        """
        blob = self.fixture.blob_3
        data = self.fixture.data_32
        mock_user = create_mock_user(1)
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]

        with self.assertRaises(AccessControlError):
            blob_api.remove_metadata(blob, data, mock_user)


def _create_user(user_id, is_superuser=False):
    """_create_user

    Args:
        user_id:
        is_superuser:

    Returns:

    """
    return create_mock_user(user_id, is_superuser=is_superuser)



""" Integration Test for User Preferences
"""
from django.core.exceptions import ValidationError
from tests.components.user_preferences.fixtures.fixtures import (
    UserPreferencesFixtures,
)

from core_main_app.commons import exceptions
from core_main_app.components.user_preferences import (
    api as user_preferences_api,
)
from core_main_app.components.user_preferences.models import UserPreferences
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    IntegrationTransactionTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user

fixture_user_preferences = UserPreferencesFixtures()


class TestUserPreferencesUpsert(IntegrationTransactionTestCase):
    """Test User Preferences Upsert"""

    fixture = fixture_user_preferences

    def test_upsert_user_preferences_creates_preferences(self):
        """test_upsert_user_preferences_creates_preferences

        Args:

        Returns:

        """
        # Arrange
        user = create_mock_user("4")

        user_preferences = UserPreferences(user_id="4", timezone="GMT")
        # Act
        result = user_preferences_api.upsert(user_preferences, user)

        # Assert
        self.assertEquals(result.timezone, user_preferences.timezone)
        self.assertEquals(result.user_id, user.id)

    def test_upsert_user_preferences_updates_preferences(self):
        """test_upsert_user_preferences_updates_preferences

        Args:

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        user_preferences = self.fixture.user_preferences_1
        user_preferences.timezone = "Europe/Paris"
        # Act
        result = user_preferences_api.upsert(user_preferences, user)

        # Assert
        self.assertEquals(result.timezone, "Europe/Paris")
        self.assertEquals(result.user_id, user.id)

    def test_upsert_user_preferences_raises_error_when_user_had_already_preferences(
        self,
    ):
        """test_upsert_user_preferences_raises_error_when_user_had_already_preferences

        Args:

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        user_preferences = UserPreferences(user_id="1", timezone="GMT")

        # Act  # Assert
        with self.assertRaises(ValidationError):
            user_preferences_api.upsert(user_preferences, user)


class TestUserPreferencesDelete(IntegrationTransactionTestCase):
    """Test User Preferences Delete"""

    fixture = fixture_user_preferences

    def test_delete_user_preferences_delete_preferences(self):
        """test_delete_user_preferences_delete_preferences

        Args:

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        user_preferences_api.delete(self.fixture.user_preferences_1, user)

        self.assertEquals(self.fixture.user_preferences_1.id, None)


class TestUserPreferencesGetByUser(IntegrationTransactionTestCase):
    """Test User Preferences Get By User"""

    fixture = fixture_user_preferences

    def test_get_user_preferences_by_user_as_owner_returns_preferences(self):
        """test_get_user_preferences_by_user_as_owner_returns_preferences

        Args:

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        result = user_preferences_api.get_by_user(user)

        # Assert
        self.assertEquals(result.timezone, "UTC")
        self.assertEquals(result.user_id, user.id)

    def test_get_user_preferences_by_user_raises_dne_exception_when_no_preferences_founded(
        self,
    ):
        """test_get_user_preferences_by_user_raises_dne_exception_when_no_preferences_founded

        Args:

        Returns:

        """

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            user_preferences_api.get_by_user(create_mock_user("0"))



""" Constants
"""

MARKDOWN_UNSAFE = 0
MARKDOWN_GENERATION_FAILED = 1

MARKDOWN_ERRORS = [
    "Unprotected script tags have been detected. Please remove them to be able to save your work!",
    "HTML is not generated properly, please check your Markdown and HTML syntax!",
]

UNKNOWN_ERROR = "An unknown error occurred, please contact your administrator for more information."

DATA_JSON_FIELD = "dict_content"

AVAILABLE_BOOTSTRAP_VERSIONS = ["4.6.2", "5.1.3", "5.3.1"]



""" Unit test for rest views
"""
from unittest import TestCase
from unittest.mock import patch, MagicMock

from django.test import override_settings, tag
from importlib_metadata import PackageNotFoundError

from core_main_app.rest.views import CoreSettings
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestCoreSetting(TestCase):
    """TestCoreSetting"""

    def test_anonymous_user_access_denied(self):
        """test_anonymous_user_access_denied

        Returns:

        """
        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            None,
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_user_get_response(self):
        """test_user_get_response

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            mock_user,
        )

        # Assert
        self.assertEqual(response.status_code, 200)

    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.utils.databases.mongo.MONGO_CLIENT", MagicMock())
    @tag("mongodb")
    def test_user_get_mongodb_info(self):
        """test_user_get_mongodb_info

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            mock_user,
        )

        # Assert
        self.assertEqual(response.data["mongodb"]["data_indexing"], True)

    def test_user_get_psql_info(self):
        """test_user_get_psql_info

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            mock_user,
        )

        # Assert
        self.assertEqual(response.data["database"]["engine"], "PostgreSQL")

    @tag("sqlite3")
    def test_user_get_sqlite_info(self):
        """test_user_get_sqlite_info

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            mock_user,
        )

        # Assert
        self.assertEqual(response.data["database"]["engine"], "SQLite3")

    @patch("core_main_app.utils.databases.backend.get_default_database_engine")
    def test_user_get_info_of_other_engine_returns_none(
        self,
        get_default_database_engine,
    ):
        """test_user_get_info_of_other_engine_returns_none

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")
        get_default_database_engine.return_value = "Other"

        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            mock_user,
        )

        # Assert
        self.assertEqual(response.data["database"]["engine"], None)

    @patch("importlib_metadata.version")
    def test_main_app_not_found_returns_none(self, version):
        """test_main_app_not_found_returns_none

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")
        version.side_effect = PackageNotFoundError()

        # Act
        response = RequestMock.do_request_get(
            CoreSettings.as_view(),
            mock_user,
        )

        # Assert
        self.assertEqual(response.data["core_version"], None)



"""
    Serializers used throughout the Rest API
"""
from os.path import join
from urllib.parse import urljoin

from django.http import Http404
from django.urls import reverse
from rest_framework.fields import CharField, ModelField
from rest_framework.fields import SerializerMethodField
from rest_framework.serializers import ModelSerializer

from core_main_app import settings
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.blob import api as blob_api
from core_main_app.components.blob.models import Blob
from core_main_app.components.blob.utils import get_blob_download_uri
from core_main_app.utils.validation.regex_validation import validate_filename


class BlobSerializer(ModelSerializer):
    """Blob serializer"""

    handle = SerializerMethodField()
    upload_date = SerializerMethodField()
    filename = ModelField(
        model_field=Blob()._meta.get_field("filename"), required=False
    )

    if "core_linked_records_app" in settings.INSTALLED_APPS:
        pid = SerializerMethodField()

    class Meta:
        """Meta"""

        model = Blob
        fields = [
            "id",
            "user_id",
            "filename",
            "handle",
            "blob",
            "workspace",
            "checksum",
            "upload_date",
        ]
        read_only_fields = (
            "id",
            "user_id",
            "handle",
            "checksum",
            "workspace",
            "upload_date",
        )

        if "core_linked_records_app" in settings.INSTALLED_APPS:
            fields.append("pid")
            read_only_fields = read_only_fields + ("pid",)

    def get_pid(self, instance):
        """Return pid

        Args:
            instance:

        Returns:

        """
        # return pid  if assigned
        if "core_linked_records_app" not in settings.INSTALLED_APPS:
            return None
        else:
            from core_linked_records_app.components.blob import (
                api as linked_blob_api,
            )
            from core_linked_records_app.settings import (
                ID_PROVIDER_SYSTEM_NAME,
            )

            try:
                sub_url = reverse(
                    "core_linked_records_provider_record",
                    kwargs={"provider": ID_PROVIDER_SYSTEM_NAME, "record": ""},
                )
                blob_pid = linked_blob_api.get_pid_for_blob(str(instance.id))

                return urljoin(
                    settings.SERVER_URI, join(sub_url, blob_pid.record_name)
                )
            except Exception:
                return None

    def get_handle(self, instance):
        """Return handle

        Args:
            instance:

        Returns:

        """
        # get request from context
        request = self.context.get("request")
        # return download handle
        return get_blob_download_uri(instance, request)

    def get_upload_date(self, instance):
        """Return upload date

        Args:
            instance:

        Returns:

        """
        # Return instance generation time
        return str(instance.creation_date)

    def get_filename_from_data(self, validated_data):
        """Get filename from validated data

        Args:
            validated_data:

        Returns:

        """
        if "filename" in validated_data:
            validate_filename(validated_data["filename"])
            return validated_data["filename"]
        if hasattr(validated_data["blob"], "name"):
            return validated_data["blob"].name

    def create(self, validated_data):
        """Create and return a new `Blob` instance, given the validated data.

        Args:
            validated_data:

        Returns:

        """
        # Create blob
        blob_object = Blob(
            filename=self.get_filename_from_data(validated_data),
            user_id=str(self.context["request"].user.id),
            blob=validated_data["blob"],
        )

        # Save the blob
        blob_api.insert(blob_object, self.context["request"].user)

        return blob_object


class DeleteBlobsSerializer(ModelSerializer):
    """Delete Blob serializer."""

    id = CharField()

    class Meta:
        """Meta"""

        model = Blob
        fields = ("id",)

    def validate_id(self, id):
        """Validate id field

        Args:
            id:

        Returns:

        """
        request = self.context.get("request")
        try:
            blob_api.get_by_id(id, request.user)
        except DoesNotExist:
            raise Http404
        return id
